#pragma once

#include "CompilerSupport/span.h"
#include "PommeTypes.h"
#include <vector>
#include <istream>
#include <ostream>
#include <memory>
#include "Sound/cmixer.h"

namespace Pomme::Sound
{
	void Init();

	void Shutdown();

	struct SampledSoundInfo
	{
		int16_t nChannels;
		uint32_t nPackets;
		int16_t codecBitDepth;
		bool bigEndian;
		double sampleRate;
		bool isCompressed;
		uint32_t compressionType;
		char* dataStart;
		int compressedLength;
		int decompressedLength;
		int8_t baseNote;
		uint32_t loopStart;
		uint32_t loopEnd;
	};

	class Codec
	{
	public:
		virtual ~Codec()
		{}

		virtual int SamplesPerPacket() = 0;

		virtual int BytesPerPacket() = 0;

		virtual int AIFFBitDepth() = 0;

		virtual void Decode(const int nChannels, const std::span<const char> input, const std::span<char> output) = 0;
	};

	class MACE : public Codec
	{
	public:
		int SamplesPerPacket() override
		{ return 6; }

		int BytesPerPacket() override
		{ return 2; }

		int AIFFBitDepth() override
		{ return 8; }

		void Decode(const int nChannels, const std::span<const char> input, const std::span<char> output) override;
	};

	class IMA4 : public Codec
	{
	public:
		int SamplesPerPacket() override
		{ return 64; }

		int BytesPerPacket() override
		{ return 34; }

		int AIFFBitDepth() override
		{ return 16; }

		void Decode(const int nChannels, const std::span<const char> input, const std::span<char> output) override;
	};

	class xlaw : public Codec
	{
		const int16_t* xlawToPCM;
	public:
		xlaw(uint32_t codecFourCC);

		int SamplesPerPacket() override
		{ return 1; }

		int BytesPerPacket() override
		{ return 1; }

		int AIFFBitDepth() override
		{ return 8; }

		void Decode(const int nChannels, const std::span<const char> input, const std::span<char> output) override;
	};

	void GetSoundInfo(const Ptr sndhdr, SampledSoundInfo& info);

	void GetSoundInfoFromSndResource(Handle sndHandle, SampledSoundInfo& info);

	std::streampos GetSoundInfoFromAIFF(std::istream& input, SampledSoundInfo& info);

	std::unique_ptr<Pomme::Sound::Codec> GetCodec(uint32_t fourCC);
}
